@@ -12,11 +12,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron.agent.common import ovs_lib
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
     as ovs_consts
 from neutron_lib import exceptions
 from oslo_log import log as logging
-import six
 
 from networking_sfc._i18n import _
 
@@ -45,32 +45,14 @@ def get_port_mask(min_port, max_port):
     return masks
 
 
-def decorate_run_ofctl(f, sfc_ovs_bridge):
-    @six.wraps(f)
-    def run_ofctl(cmd, args, process_input=None):
-        return f(cmd, ["-O " + sfc_ovs_bridge.of_version] + args,
-                 process_input)
-
-    return run_ofctl
-
-
-class SfcOVSBridgeExt(object):
+class SfcOVSBridgeExt():
 
     def __init__(self, ovs_bridge):
         self.bridge = ovs_bridge
-        # OpenFlow 1.1 for groups
-        self.of_version = ovs_consts.OPENFLOW11
 
-        # this is so that our own run_ofctl is used when we call e.g. add_flows
-        # (proxying is not enough, because the proxied bridge would still call
-        # its own run_ofctl):
-        ovs_bridge.run_ofctl = decorate_run_ofctl(
-            ovs_bridge.run_ofctl,
-            self)
-
-    def set_protocols(self, protocols):
-        self.of_version = protocols[-1]
-        self.bridge.set_protocols(protocols)
+        # OpenFlow 1.3 is needed to manipulate groups
+        # To support NSH feature, OpenFlow 1.3 is also needed
+        self.bridge.use_at_least_protocol(ovs_consts.OPENFLOW13)
 
     # proxy most methods to self.bridge
     def __getattr__(self, name):
@@ -78,7 +60,7 @@ class SfcOVSBridgeExt(object):
 
     def do_action_groups(self, action, kwargs_list):
         group_strs = [_build_group_expr_str(kw, action) for kw in kwargs_list]
-        if action == 'add' or action == 'del':
+        if action in ('add', 'del'):
             cmd = '%s-groups' % action
         elif action == 'mod':
             cmd = '%s-group' % action
@@ -102,9 +84,16 @@ class SfcOVSBridgeExt(object):
         group = self.run_ofctl("dump-groups", [group_str])
         if group:
             retval = '\n'.join(item for item in group.splitlines()
-                               if 'NXST' not in item)
+                               if ovs_lib.is_a_flow_line(item))
         return retval
 
+    def get_bridge_ports(self):
+        port_name_list = self.bridge.get_port_name_list()
+        of_portno_list = list()
+        for port_name in port_name_list:
+            of_portno_list.append(self.bridge.get_port_ofport(port_name))
+        return of_portno_list
+
 
 def _build_group_expr_str(group_dict, cmd):
     group_expr_arr = []
